// ================================================================
//  Educational Game Platform - with a top level Menu selection
//  using a Micro Controller, 8 LED 4 color bar and 4 Button module
//

//  include Library of Hardware Support functions
#include "Mini_STEM_Platform.h"

unsigned long t0,t1;

// the setup routine runs once on power up or when you press reset:
//=================================================================
void setup() {
  int aVal, val;
  
  // initialize serial communication at 9600 bits per second:
  Serial.begin(9600);
  Serial.println("Arduino Educational Mini Game Platform - Lesson 11");

  // ------------ setup the Mini Game/STEM platform (8LEDs, 4Btns, and Audio)
  gPlatform_setup();

  beep(50,50); beep(50,50); beep(150);  // announce "it is alive!"

  // wait an amount of pseudo random time due to HW input noise (Entropy), then randomize
  aVal = val = analogRead(A0);
  while (abs(aVal-val)<=20) {val=analogRead(A0); delayMicroseconds(micros()%val);}    // expects analog noise (A0 needs to be open)
  randomSeed(micros()/4);  // micros() resolution is ~ 4us
}

// the loop routine runs over and over again forever:
//=================================================================
void loop() { // Top Menu level, for selecting a activity
  while(1) {
    clearDisp();
    while(Btn) {buZZ(1); scanBtns();}  // wait for no button

    while(!Btn) {
      for (int i=1; i<=nleds; i++) {  // flash odd & even LEDs in opposition while waiting 
        dim[i] = ((msCnt/333+i)%2);   // every 1/3 sec switch which LEDs are dim / lit
      }
      refreshDisp();
      delay(1);
      scanBtns();
    }
    Serial.print("Function: ");
    Serial.println(btnNum);
    
    clearDisp();
//    if (btn1) HW_Support_Demo();   // Demonstrates LED display and audio support functionality
//    else if (btn2) Speed_Scan_ver4();   // hit it on the right spot to speed things up to a winning max
    if (btn1) Magic_8_ball();
    if (btn2) Magic_8_ball_v2();
    else if (btn3) React_Now_ver2();    // Test your Reaction time 
    else if (btn4) btnReporting();    // exercises the buttons and Leds, and reports via Serial 
               //  Or replace this with your own previously devised activity   }
  }
}

// Various selection of mini APPs / Games
//=================================================================

// ------------------
void btnReporting() {
  // =================================  Button - LED exercise & Reporting
  int icnt=0; // iteration count, lsb~=1msec
  clearDisp();
  while (Btn) scanBtns();  // wait for button release
  while (1) {
    refreshWait(1);
    icnt++;
    scanBtns();
    // ------------------- button reporting
    if (BtnChanged) {
      Serial.println(" ");
      if (btn1) Serial.print("[Btn1] "); else Serial.print("       ");
      if (btn2) Serial.print("[Btn2] "); else Serial.print("       ");
      if (btn3) Serial.print("[Btn3] "); else Serial.print("       ");
      if (btn4) Serial.print("[Btn4] "); else Serial.print("       ");
      Serial.print(btnState);

      // ------------------ LED reflection of button state
      lit[1] = lit[2] = btn1; // set Red LEDs
      lit[3] = lit[4] = btn2; // set Yel LEDs
      lit[5] = lit[6] = btn3; // set Blu LEDs
      lit[7] = lit[8] = btn4; // set Grn LEDs
      brightOne=0;

      beep(15); // make a key press click, for up&down
      icnt=0;
    } else if (icnt>10000) {  // if ignorned for 10 seconds
      icnt=0;
      brightOne=1;
      beep(15,200);
      clearDisp();
    }
    if (ESC && brightOne==0 && btn4 ) Serial.print(" [ESC] ");    
    brightOne = (ESC)? 8 : 0;
    if (ESC && btn1) break;    // only leave Btn Reporting if btn1 is also held.
  }
}

void HW_Support_Demo() {  // Demonstrates use of set of Hardware support functions
  while(true) {
    scanBtns();

    if (BtnPressed) {
      clearDisp();
      for (int i=1; i<=nleds; i++) {  // set all LEDs to a given level
        if (btn1Pressed) dim[i]=true;
        if (btn2Pressed) lit[i]=true;
        if (btn3Pressed) brt[i]=true;        
      }
    }
    //if (btn4Pressed) {clearDisp();}   // not needed
    
    // produce sound while btn1-3 is held
    while (Btn) {
      if (btn1 || btn3) boop(2);
      if (btn2 || btn3) beep(2);
      if (btn4) break;  // don't get stuck here
      scanBtns();
    }
    
    refreshDisp();
    delay(1);
//
    if (ESC) break;
  } 
}

void Speed_Scan_ver4() {  // hit it on the right spot to speed things up to a winning max
  int onMSecs, stRate=150;
  int hits=0, goal=9;

  litLED=1;
  onMSecs=stRate;
  while(true) { 
    refreshWait(onMSecs);     // flash the LED (blip) for a bit

    // allow use of left or right most button
    scanBtns();

    if (BtnPressed && (litLED==5 || litLED==6)) {
        beep(100);    
        hits++;
        if (hits==goal) {
          beep(100,50); beep(100,50); beep(250);  // you did it
          hits=0; // start over
          refreshWait(1000);
          clearDisp();
        }
    }

    // added block for support of a missed penalty
    if (BtnPressed && !(litLED==5 || litLED==6)) {
      refreshWait(300);
      hits--;     // slow down update rate
      clearDisp();
    }
    
    onMSecs = stRate - (15*hits); // more hits he get faster the blip scans
    for (int i=1; i<=hits; i++) dim[i]=true;    // display as, dim, the hit progress
    litLED = (litLED % nleds) + 1;    // advance to next LED

    if (ESC) break;
  }
}

//----------------  React_Now - Swat Quickly as LEDs light up
void React_Now_ver2() { // attemp to hit the button when the first LED lights up
                   // 1st LED is lit for 100 msecs, additional LEDs lite every 20ms
                   // challenge: if one pushes & holds before the first LED get wrong win; how to fix ?
  int min_Period = 100;
  int LEDptr;
  
  while(true) {
    // insure that the button is not already pressed
    while (Btn && !ESC) scanBtns();
    if (ESC) return;
    
    // insure all LEDs are off
    clearDisp();

    delay(random(2000,5000));       // wait for 2 - 5 seconds
    lit[1]=1;
    refreshDisp();
    beep(10); // takes 2x msecs
    delay(min_Period-20-20);    // account for the 20ms beep time and 20ms in for loop

    for (LEDptr=1; LEDptr<=8; LEDptr++) {
      lit[LEDptr]=true;
      refreshWait(20);
      scanBtns();
      if (BtnPressed) { // they pressed the button
        brightOne=LEDptr;
        refreshWait(1000);
        break;
      }
    }
    if (LEDptr>8) beep(100);
    if (ESC) break;
  }
}


// ==========================================  Magic-8-Ball
// hold down btn1 ... ponder question, let go.  Red: NO,  YEL: Maybe,  Grn: YES  Blue: Try again
// use btn2 for 1/6 out comes to use as a dice, useful for games.  (use btn3 for 1/8)

void Magic_8_ball() {
  int cnt;
  byte ans, keyCode, color;
  byte priorBtn;
  
  while(Btn) scanBtns(); // wait for previous button press to be released
  
  while (1) {
    clearDisp();
    while(!Btn) {
      scanBtns();
      litLED=(msCnt/1000)%2;    // toggles first LED once per second
      refreshWait(1);
    }
    if (ESC || btn4) break;
    
    t0 = millis();  // start timing on Button pushed, 
       // then use human originated action Entropy to directly effect the resulting answer
    while(Btn) {  // wait for release
      priorBtn=btnNum;
      scanBtns();
      if (cnt != (msCnt/400)%8) {  // flash LEDs (1-3-2-4)
        clearDisp();
        // determine next color set to light or not to light
        cnt = (msCnt/400)%8;
        color=cnt;
        if (cnt==5 || cnt==7) color=cnt-3;
        if ((cnt%2)==0) color=0;
        showColor(color);
      }
      refreshWait(1); 
    }
    
    keyCode = priorBtn;
    clearDisp();
    refreshWait(300 + random(900));  // make them wait alittle
                                     // this also influences, randomly, the results
    if (keyCode==1) {            // select one of the 4 colors
      ans = (millis() - t0) %4;  // answer is determined by their action
      color=ans+1;
      showColor(color);
    } else if (keyCode==2) {     // select one of the 6 LEDs , can be used as die for for game dice 
      ans = (millis() - t0) %6;  // answer is determined by their action
      brightOne = ans+1;
    } else {                     // btn3 was used, 1 of N leds
      ans = (millis() - t0) %nleds;
      brightOne = ans+1;
    }

    while(1) {      // await the next Request
      scanBtns();
      refreshWait(1);
      if (ESC) return;     // ESC when done
      if (Btn) break;      // go again 
    }
  }
}

// hold down btn1 ... ponder question, let go.  Red: NO,  YEL: Maybe,  Grn: YES  Blue: Try again
// use btn2 for 1/6 out comes ~ like a dice, useful for games.  (use btn3 for 1/8)
void Magic_8_ball_v2() {
  byte ans, keyCode, color;
  byte priorBtn;
  static byte ccycle[] = {0,1,0,3,0,2,0,4};
  static byte dicePat[] = {0x10, 0x42, 0x2A, 0xA5, 0xD3, 0xE7};
  
  while(Btn) scanBtns(); // wait for previous button press to be released
  
  while (1) {
    clearDisp();
    while(!Btn) {
      scanBtns();
      litLED=(msCnt/1000)%2;    // toggles 0/1 per second
      refreshWait(1);
    }
    if (ESC || btn4) break;
    
    t0 = millis();  // start timing on  Button pushed
    litLED=0;
    
    while(Btn) {  // wait for release
      priorBtn=btnNum;
      scanBtns();
      if (color != (msCnt/333)%8) {
        // determine next color set to light or not to light
        color = (msCnt/333)%8;
        clearDisp();
        showColor(ccycle[color]);
      }  // flash odd & even LEDs in opposition
      refreshWait(1);
    }
    
    keyCode = priorBtn;
    clearDisp();
    refreshWait(200 + random(800));  // make them wait alittle
    
    if (keyCode==1) {            // select one of the 4 colors
      ans = (millis() - t0) %4;  // answer is determined by their action
      showColor(ans+1);
    } else if (keyCode==2) {     // select one of the 6, can be used as die for for game dice 
      ans = (millis() - t0) %6;  // answer is determined by their action
//      brightOne = ans+1;
      set_LEDs(dicePat[ans],2);   // lite a given pattern, similar to dice, for 1-6
    } else {                     // btn3 was used, 1 of N leds
      ans = (millis() - t0) %nleds;
      brightOne = ans+1;
    }

    while(1) {      // await the next Request
      scanBtns();
      refreshWait(1);
      if (ESC) return;     // ESC when done
      if (Btn) break;      // go again 
    }
  }
}

// ================== Local Support Utility Functions =================

void set_LEDs(byte led_pattern, byte state) {
  // led_pattern: bit decoded pattern, eg: 9= (00001001b)left most led + forth (led#1 & #4)
  // state: 1=dim, 2=lit, 3=bright, 0=clear
  byte b;
  for (int i=1; i<=8; i++) {
    b = led_pattern & 1;
    led_pattern=led_pattern>>1;
    if (b) {
      dim[i]=0; lit[i]=0; brt[i]=0; // default state cleared
      if (state==1)  dim[i] = true;
      else if (state==2)  lit[i] = true;
      else if (state==3)  brt[i] = true;
    }
  }
}

void showColor(byte color) {
  if (color==0) return;
  if (color==1 || color==5)  lit[1] = lit[2] = true; // 1 Red LEDs
  if (color==2 || color==5)  lit[3] = lit[4] = true; // 2 Yel LEDs
  if (color==3 || color==5)  lit[5] = lit[6] = true; // 3 Blu LEDs
  if (color==4 || color==5)  lit[7] = lit[8] = true; // 4 Grn LEDs
}

// -----------------------
void showLevel(byte level) {
  int n;
  clearDisp();
  refreshWait(250);
  for (int i=0; i<level; i++) {
    beep(10);
    n = (i%nleds)+1;  // n wraps around as i exceeds nleds
    dim[n]=true;
    brightOne=n;
    refreshWait(100);
  }
}
